<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 05/29/2004 *
 * ===================                          Last change: 01/07/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : ext-autopurge.php                                *
 * -------------------------------------------------------------------- *
 * Short description : Automatical purging of outdated mail links       *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Auto-Loeschung von veralteten Mail-Links         *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Version number
setThisExtensionVersion('0.4.1');

// Version history array (add more with , '0.0.1' and so on)
setExtensionVersionHistory(array('0.0.0', '0.1.0', '0.1.1', '0.1.2', '0.1.3', '0.1.4', '0.1.5', '0.1.6', '0.1.7', '0.1.8', '0.1.9', '0.2.0', '0.2.1', '0.2.2', '0.2.3', '0.2.4', '0.2.5', '0.2.6', '0.2.7', '0.2.8', '0.2.9', '0.3.0', '0.3.1', '0.3.2', '0.3.3', '0.3.4', '0.3.5', '0.3.6', '0.3.7', '0.3.8', '0.3.9', '0.4.0', '0.4.1'));

// Keep this extension always active!
setExtensionAlwaysActive('Y');

switch (getExtensionMode()) {
	case 'register': // Do stuff when installation is running
		// Add dependency to ext-bonus
		addExtensionDependency('bonus');

		// SQL commands to run
		addExtensionChangeTableColumnSql('pool', 'data_type', 'data_type', "ENUM('TEMP','SEND','NEW','ADMIN','ACTIVE','DELETED') NOT NULL DEFAULT 'TEMP'");
		addExtensionChangeTableColumnSql('bonus', 'data_type', 'data_type', "ENUM('NEW','QUEUE','SEND','DELETED') NOT NULL DEFAULT 'NEW'");
		break;

	case 'remove': // Do stuff when removing extension
		// SQL commands to run
		addExtensionChangeTableColumnSql('pool', 'data_type', 'data_type', "ENUM('TEMP','SEND','NEW','ADMIN','ACTIVE') NOT NULL DEFAULT 'TEMP'");
		addExtensionChangeTableColumnSql('bonus', 'data_type', 'data_type', "ENUM('NEW','QUEUE','SEND') NOT NULL DEFAULT 'NEW'");
		addExtensionSql("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_admin_menu` WHERE `what` IN('config_autopurge','list_autopurge')");
		addExtensionDropTableColumnSql('user_data', 'ap_notified');

		// Unregister with points data
		unregisterExtensionPointsData('autopurge_add');
		break;

	case 'activate': // Do stuff when admin activates this extension
		// SQL commands to run
		addExtensionSql('');
		break;

	case 'deactivate': // Do stuff when admin deactivates this extension
		// SQL commands to run
		addExtensionSql('');
		break;

	case 'update': // Update an extension
		switch (getCurrentExtensionVersion()) {
			case '0.1.0': // SQL queries for v0.1.0
				// Add dependency to 'user'
				addExtensionDependency('user');

				// SQL queries
				addConfigAddSql('autopurge_inactive', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('autopurge_unconfirmed', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_inactive_since', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT 2592000');
				addConfigAddSql('ap_inactive_time', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT 25200');
				addConfigAddSql('ap_unconfirmed_time', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT 432000');
				addAdminMenuSql('setup', 'config_autopurge', 'Auto-L&ouml;schung', 'Automatisch inaktive oder nicht best&auml;tigte Accounts l&ouml;schen.', 12);
				addAdminMenuSql('user', 'list_autopurge', 'Inaktive finden', 'Lassen Sie sich vor dem t&auml;glichen Reset anzeigen, welche Mitglieder als inaktiv erkannt werden und welche gel&ouml;scht werden.', 10);
				addExtensionAddTableColumnSql('user_data', 'ap_notified', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT 0');

				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Automatisches L&ouml;schen von inaktiven bzw. nicht best&auml;tigten Accounts hinzugef&uuml;gt.");
				break;

			case '0.1.2': // SQL queries for v0.1.2
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Automatisches L&ouml;schen von als zu l&ouml;schen markierte Tasks hinzugef&uuml;gt.");
				break;

			case '0.1.3': // SQL queries for v0.1.3
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Nicht mehr verwendetes Update.");
				break;

			case '0.1.4': // SQL queries for v0.1.4
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Fehler <div class=\"notice\">Warning: Missing argument 2 for create_timestamp_from_selections() in {?PATH?}inc/libs/pro_functions.php on line 227</div> behoben.");
				break;

			case '0.1.5': // SQL queries for v0.1.5
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Anstelle von <strong>ref_depth</strong> wurde <strong>level</strong> programmiert.");
				break;

			case '0.1.6': // SQL queries for v0.1.6
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Fehler beseitigt, wenn error_reporting=E_ALL gesetzt ist.");
				break;

			case '0.1.7': // SQL queries for v0.1.7
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Fehler beseitigt, wenn error_reporting=E_ALL gesetzt ist.");
				break;

			case '0.1.8': // SQL queries for v0.1.8
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Problem mit Speicherung der Einstellungen beseitigt.");
				break;

			case '0.1.9': // SQL queries for v0.1.9
				addConfigAddSql('autopurge_tasks', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_tasks_time', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT ' . (getOneDay() * 7));

				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Bereinigung von zu l&ouml;schenden Aufgaben klappt wieder. Zeitlimit f&uuml;r genanntes kann eingestellt werden (Default = 7 Tage).<br /><br /><u>Bitte aktualisieren Sie auch die Admin-Templates!</u>");
				break;

			case '0.2.0': // SQL queries for v0.2.0
				addConfigAddSql('ap_in_notify', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_un_notify', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_tasks_notify', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");

				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Mail wird bei L&ouml;schung von Aufgaben ausgesendet.<br /><br /><u>Bitte aktualisieren Sie auch die Admin-Templates!</u>");
				break;

			case '0.2.1': // SQL queries for v0.2.1
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Diverse angebundene Erweiterungen gefixt.");
				break;

			case '0.2.2': // SQL queries for v0.2.2
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Betreffzeile f&uuml;r gel&ouml;schte Aufgaben korregiert.");
				break;

			case '0.2.3': // SQL queries for v0.2.3
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Erweiterung bleibt wegen integrierten Schalters immer aktiv.");
				break;

			case '0.2.4': // SQL queries for v0.2.4
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Auto-L&ouml;schung von Best&auml;tigungslinks kann unter &quot;Sonstige Einstellungen&quot; abgeschaltet werden (0 setzen!)");
				break;

			case '0.2.5': // SQL queries for v0.2.5
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Seit <strong>Patch 340</strong> &uuml;berfl&uuml;ssige HTML-Tags entfernt.");
				break;

			case '0.2.6': // SQL queries for v0.2.6
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Problem mit Jackpot beseitigt ({?POINTS?} wurden nicht gutgeschrieben.)");
				break;

			case '0.2.7': // SQL queries for v0.2.7
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Link zum Mitgliedsprofil in Funktion <u>generateUserProfileLink()</u> ausgelagert.");
				break;

			case '0.2.8': // SQL queries for v0.2.8
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Link zum Mitgliedsprofil in Funktion <u>generateUserProfileLink()</u> ausgelagert.");
				break;

			case '0.2.9': // SQL queries for v0.2.9
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Mailbetreffs korregiert.");
				break;

			case '0.3.0': // SQL queries for v0.3.0
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Template <u>admin_config_autopurge_pro.tpl</u> ist &uuml;berfl&uuml;ssig geworden. Bitte l&ouml;schen Sie dies!");
				break;

			case '0.3.1': // SQL queries for v0.3.1
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Datumsformat festgelegt auf ausf&uuml;hrlich.");
				break;

			case '0.3.2': // SQL queries for v0.3.2
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("W&ouml;rter <strong>{?mt_word?}</strong>, <strong>{?mt_word2?}</strong> und <strong>{?mt_word3?}</strong> sind austauschbar.");
				break;

			case '0.3.3': // SQL queries for v0.3.3
				addConfigAddSql('ap_del_mails', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_dm_notify', "ENUM('Y','N') NOT NULL DEFAULT 'Y'");
				addConfigAddSql('ap_dm_timeout', 'BIGINT(20) UNSIGNED NOT NULL DEFAULT {?ONE_DAY?}');

				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Von bereits gel&ouml;schten Mitgliedern die Mails l&ouml;schen integriert.");
				break;

			case '0.3.4': // SQL queries for v0.3.4
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Fehler mit <strong>timestamp_send</strong> (Zeile 308) beseitigt.");
				break;

			case '0.3.5': // SQL queries for v0.3.5
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Fehler mit <strong>ap_del_emails</strong> (Einstellungen im Admin-Bereich) beseitigt.");
				break;

			case '0.3.6': // SQL queries for v0.3.6
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Die Accounts der Standard-Referral-Id, der Bonus-Id, Bettellink-Id und der Verdoppler-Id werden nun nicht mehr gel&ouml;scht und tauchen auch unterhalb der Inaktiven-Liste nicht mehr auf.");
				break;

			case '0.3.7': // SQL queries for v0.3.7
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Abspeichern von Einstellungen repariert.");
				break;

			case '0.3.8': // SQL queries for v0.3.8
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Vorbereitung auf die neue Mediendaten v0.0.4.");
				break;

			case '0.3.9': // SQL queries for v0.3.9
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("if-Anweisungen auf Funktion <strong>empty()</strong> umgestellt. Unter <strong>Auto-L&ouml;schung</strong> finden Sie nun auch die Einstellungen zu Mailbest&auml;tigungs wieder. Zudem werden keine Urlauber (neue Urlaubsschaltung beachtet) mehr als inaktiv erkannt.");
				break;

			case '0.4.0': // SQL queries for v0.4.0
				// Update notes (these will be set as task text!)
				setExtensionUpdateNotes("Template-Problem beseitigt. Dies verhinderte das Abspeichern der Einstellungen.");
				break;

			case '0.4.1': // SQL queries for v0.4.1
				// Register with points data system
				registerExtensionPointsData('autopurge_add', 'points', 'LOCKED', 'DIRECT');

				// Update notes
				setExtensionUpdateNotes("R&uuml;ckschriften bei Auto-L&ouml;schungen werden nun &uuml;ber die Tabelle <strong>{OPEN_CONFIG}_MYSQL_PREFIX{CLOSE_CONFIG}_points_data</strong> verwaltet.");
				break;
		} // END - switch
		break;

	case 'modify': // When the extension got modified
		break;

	case 'test': // For testing purposes
		break;

	case 'init': // Do stuff when extension is initialized
		break;

	default: // Unknown extension mode
		reportBug(__FILE__, __LINE__, sprintf("Unknown extension mode %s in extension %s detected.", getExtensionMode(), getCurrentExtensionName()));
		break;
} // END - switch

// [EOF]
?>
