<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 11/10/2004 *
 * ===================                          Last change: 03/18/2005 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : bonus_functions.php                              *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for bonus extension            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktion fuer bonus-Erweiterung        *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// This function must be run *BEFORE* a link is removed from table 'mailer_user_links' !
function addTurboBonus ($id, $userid, $type) {
	// Shall we add bonus points?
	if (!isBonusRallyeActive()) {
		return FALSE;
	} // END - if

	// Init variables
	$sql     = '';
	$bonusId = 'NULL';
	$mailId  = 'NULL';
	$column  = '';

	// Select SQL command
	switch ($type) {
		case 'bonusid':
			$column = 'bonus_id';
			$bonusId = $id;
			break;

		case 'mailid' :
			$column = 'mail_id';
			$mailId = $id;
			break;

		default:
			logDebugMessage(__FUNCTION__, __LINE__, sprintf("Invalid type %s detected.", $type));
			break;
	} // END - switch

	// Is a column name set?
	if (empty($column)) {
		// No, then abort here
		return FALSE;
	} // END - if

	// Check for entry
	$rank = countSumTotalData($id, 'bonus_turbo', 'id', $column, TRUE) + 1;

	// Which rank?
	if ($rank == 1) {
		// First rank!
		$points = getTurboBonus();
	} else {
		// Anything else so let's explode all entered rank points
		$test = explode(';', getTurboRates());
		if (!empty($test[$rank - 2])) {
			// Level found
			$points = $test[$rank - 2];
		} else {
			// Level not found
			$points = '0.00000';
		}
	}

	// Add points to his account directly
	SQL_QUERY_ESC("UPDATE `{?_MYSQL_PREFIX?}_user_data` SET `turbo_bonus`=`turbo_bonus`+%s WHERE `userid`=%s LIMIT 1",
		array(
			$points,
			bigintval($userid)
		), __FUNCTION__, __LINE__);

	// Rember this whole data for displaying ranking list
	SQL_QUERY_ESC("INSERT INTO `{?_MYSQL_PREFIX?}_bonus_turbo` (`userid`, `mail_id`, `bonus_id`, `level`, `points`, `timemark`) VALUES (%s, %s, %s, %s, %s, UNIX_TIMESTAMP())",
		array(
			bigintval($userid),
			$mailId,
			$bonusId,
			$rank,
			$points
		), __FUNCTION__, __LINE__);

	// @TODO Rewrite this to a filter
	if ((isExtensionInstalledAndNewer('bonus', '0.3.5')) && (getBonusMode() != 'ADD') && ($points > 0)) {
		handleBonusPoints($points, $userid);
	} // END - if
}

//
function addBonusRanks ($data, $type, $userid) {
	// Init variables
	$self = FALSE; $OUT = ''; $GLOBALS['ranking_content'] = array();

	// Clear rankings by default
	$GLOBALS['ranking_content']['rankings'] = '';

	// How many ranks do we have?
	$ranks = count(explode(';', getTurboRates())) + 1;

	// Load current user's data
	$result = SQL_QUERY_ESC("SELECT `level`, `points`, `timemark` FROM `{?_MYSQL_PREFIX?}_bonus_turbo` WHERE `%s`=%s AND `userid`=%s LIMIT 1",
		array(
			$type,
			$data,
			bigintval($userid)
		), __FUNCTION__, __LINE__);

	// Entry found?
	if (SQL_NUMROWS($result) == 1) {
		// Load data
		$GLOBALS['ranking_content'] = merge_array($GLOBALS['ranking_content'], SQL_FETCHARRAY($result));

		// Remember all values for later use
		$self  = TRUE;

		// Transfer data to template
		$GLOBALS['ranking_content']['timemark'] = generateDateTime($GLOBALS['ranking_content']['timemark'], 1);

		// Load template
		$GLOBALS['ranking_content']['own'] = loadTemplate('show_bonus_yr', TRUE, $GLOBALS['ranking_content']);
	} // END - if

	// Load rankings
	$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_bonus_turbo` WHERE `%s`=%s ORDER BY `level` ASC LIMIT {?bonus_lines?}",
		array($type, $data), __FUNCTION__, __LINE__);
	if (!SQL_HASZERONUMS($result)) {
		// Output all ranks (levels)
		for ($rank = 1; $rank <= SQL_NUMROWS($result); $rank++) {
			// Load data
			$result_users = SQL_QUERY_ESC("SELECT
	`userid`,
	`points`
FROM
	`{?_MYSQL_PREFIX?}_bonus_turbo`
WHERE
	`%s`=%s AND
	`level`=%s
LIMIT 1",
				array($type, $data, $rank), __FUNCTION__, __LINE__);

			// Nothing found by default
			$rows['userid'] = '---';
			$rows['points'] = '---';

			// Are you one of them?
			if (SQL_NUMROWS($result_users) == 1) {
				// Load data
				$rows = merge_array($rows, SQL_FETCHARRAY($result_users));
			} // END - if

			// Free result
			SQL_FREERESULT($result_users);

			// Add more
			$rows['rank'] = $rank;

			// Load row template
			$OUT .= loadTemplate('member_bonus_turbo_row', TRUE, $rows);
		} // END - for

		if ($self === FALSE) {
			// If current user was not found set constant
			$GLOBALS['ranking_content']['rankings'] = '{--MEMBER_BONUS_RANK_YOU_ARE_404--}';
		} // END - if
	} else {
		// No entries found
		// @TODO Move this HTML to a template
		$OUT = '<tr>
  <td colspan="3" align="center" height="30" class="bottom">
    <div class="bad">{%message,MEMBER_BONUS_NO_RANKS=' . $data . '%}</div>
  </td>
</tr>';
	}

	// Retutn content
	return $OUT;
}

// Hanle any bonus points the given user shall become
function handleBonusPoints ($mode, $userid) {
	// Shall we add bonus points?
	if (!isBonusRallyeActive()) {
		return;
	} // END - if

	// Default is not working
	$return = FALSE;

	// Switch to jackpot-mode when no UID is supplied but userid-mode is selected
	if ((getBonusMode() == 'UID') && (!isValidId($userid)) && (isExtensionActive('jackpot'))) {
		// Update database & config
		updateConfiguration('bonus_mode', 'JACKPOT');
	} // END - if

	if ($mode == 'login_bonus') {
		// Login bonus detected
		$points = getLoginBonus();
	} else {
		// Direct points supplied
		$points = $mode;
	}

	// Check his amount first
	$total = getTotalPoints($userid);

	// Subtract points from...
	switch (getBonusMode()) {
		case 'ADD': // Only add them (no subtraction)
			// Ignored
			break;

		case 'JACKPOT': // ... jackpot
			if ((isExtensionActive('jackpot')) && (subtractPointsFromJackpot($points) === FALSE) && (isValidId($userid))) {
				if ($total >= $points) {
					// Subtract points from userid's account
					$return = subtractPointsFromJackpot('bonus_payout_jackpot', $userid, $points);
				} // END - if
			} // END - if
			break;

		case 'UID': // ... userid's account
			if ($total >= $points) {
				// Subtract points from userid's account
				$return = subtractPoints('bonus_payout_userid', $userid, $points);
			} elseif (isExtensionActive('jackpot')) {
				// Try to subtract from jackpot
				$return = subtractPointsFromJackpot($points);
			}
			break;

		default: // This should not happen
			reportBug(__FUNCTION__, __LINE__, 'Invalid bonus-mode ' . getBonusMode() . ' detected.');
			break;
	} // END - switch

	// Return status
	return $return;
}

// Purges expired fast-click bonus entries
function purgeExpiredTurboBonus() {
	// Remove entries
	$result = SQL_QUERY('DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_bonus_turbo` WHERE (UNIX_TIMESTAMP() - `timemark`) >= {?bonus_timeout?}', __FUNCTION__, __LINE__);

	if (!SQL_HASZEROAFFECTED()) {
		// Send out email to admin
		sendAdminNotification('{--ADMIN_AUTOPURGE_TURBO_SUBJECT--}', 'admin_autopurge_turbo', SQL_AFFECTEDROWS());
	} // END - if
}

//-----------------------------------------------------------------------------
//                             Wrapper Functions
//-----------------------------------------------------------------------------

// Determines whether the "bonus rallye" is active
function isBonusRallyeActive () {
	// Is there cache?
	if (!isset($GLOBALS['bonus_rallye_active'])) {
		// Just determine it
		$GLOBALS['bonus_rallye_active'] = (getBonusActive() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS['bonus_rallye_active'];
}

// Determines whether the "bonus new_member_notify" is active
function isBonusNewMemberNotifyEnabled () {
	// Is there cache?
	if (!isset($GLOBALS['bonus_new_member_notify_active'])) {
		// Just determine it
		$GLOBALS['bonus_new_member_notify_active'] = (getBonusNewMemberNotify() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS['bonus_new_member_notify_active'];
}

// Getter for bonus_timeout
function getBonusTimeout () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_mode
function getBonusMode () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_mode');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_ranks
function getBonusRanks () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_ranks');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for turbo_rates
function getTurboRates () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('turbo_rates');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for login_timeout
function getLoginTimeout () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('login_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for login_bonus
function getLoginBonus () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('login_bonus');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for turbo_bonus
function getTurboBonus () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('turbo_bonus');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_ref
function getBonusRef () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_ref');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_userid
function getBonusUserid () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_userid');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_active
function getBonusActive () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_active');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_notify_points
function getBonusNotifyPoints () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_notify_points');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_new_member_notify
function getBonusNewMemberNotify () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_new_member_notify');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_stats
function getBonusStats () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_stats');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_order
function getBonusOrder () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_order');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_lines
function getBonusLines () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_lines');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_include_own
function getBonusIncludeOwn () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_include_own');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether bonus_include_own is set to 'Y'
function isBonusIncludeOwnEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getBonusIncludeOwn() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_disable_notify
function getBonusDisableNotify () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_disable_notify');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for bonus_enable_notify
function getBonusEnableNotify () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_enable_notify');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for include_bonus_click
function getIncludeBonusClick () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('include_bonus_click');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether include_bonus_click is set to 'Y'
function isIncludeBonusClickEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIncludeBonusClick() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for include_bonus_login
function getIncludeBonusLogin () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('include_bonus_login');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether include_bonus_login is set to 'Y'
function isIncludeBonusLoginEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIncludeBonusLogin() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for include_bonus_order
function getIncludeBonusOrder () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('include_bonus_order');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether include_bonus_order is set to 'Y'
function isIncludeBonusOrderEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIncludeBonusOrder() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for include_bonus_Ref
function getIncludeBonusRef () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('include_bonus_ref');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether include_bonus_ref is set to 'Y'
function isIncludeBonusRefEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIncludeBonusRef() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for include_bonus_stats
function getIncludeBonusStats () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('include_bonus_stats');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks whether include_bonus_stats is set to 'Y'
function isIncludeBonusStatsEnabled () {
	// Is there cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getIncludeBonusStats() == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
